dp!() { /* proc-macro */ }Expand description
Alias for dparagraph.
Create a paragraph with interactive elements from a string.
Interactive text sections are automatically added from text delimited by [[ and ]] (Also see: [mparagraph]).
The return type is the value of whichever text token that was clicked.
Syntax
dparagraph!(maybe_key, expr1, expr2, ..., exprN)
# Additional
Text is trimmed
Multiple inputs are accepted, and produce multiple paragraphs
The interactive elements must not change between renders.