Module view

Module view 

The View and its constituents.

Structs

Image
Image type. See crate::view::Object.
Line
A collection of Span’s, rendered in a wrapped line, joined without spacing.
Modifier
Applies a set of styles to Span.
Span
Abstract span. Similar in principle to an HTML element/egui TextFormat.
View
The view returned by a Page.

Enums

ImageVariant
Local or Remote image. See Image.
Object
An object within a View.
SpanVariant
Applies a style preset.

Type Aliases

RenderData
Some Object variants may want additional data to specify custom styles for the frontend. That can be specified here.