The View and its constituents.
Structs
- Image
- Image type.
See
crate::view::Object. - Line
- A collection of
Span’s, rendered in a wrapped line, joined without spacing. - Modifier
- Applies a set of styles to
Span. - Span
- Abstract span. Similar in principle to an HTML element/egui TextFormat.
- View
- The view returned by a
Page.
Enums
- Image
Variant - Local or Remote image.
See
Image. - Object
- An object within a
View. - Span
Variant - Applies a style preset.
Type Aliases
- Render
Data - Some
Objectvariants may want additional data to specify custom styles for the frontend. That can be specified here.